# This software is licensed under NNCL v1.2-MODIFIED-OpenShockPY see LICENSE.md for more info
# https://github.com/NanashiTheNameless/OpenShockPY/blob/main/LICENSE.md
from .client import OpenShockClient, OpenShockError

__all__ = [
    "OpenShockClient",
    "OpenShockError",
]
